@@ -17,7 +17,12 @@ sub does_role {
 
 use namespace::clean -except => 'meta';
 
-with qw(KiokuDB::TypeMap::Entry::Std);
+with (
+    'KiokuDB::TypeMap::Entry::Std',
+    'KiokuDB::TypeMap::Entry::Std::Expand' => {
+        alias => { compile_expand => 'compile_expand_body' },
+    }
+);
 
 # FIXME collapser and expaner should both be methods in Class::MOP::Class,
 # apart from the visit call
@@ -130,7 +135,7 @@ sub compile_collapse_body {
                         }
                     }
 
-                    my $value = $attr->get_value($object);
+                    my $value = $attr->get_raw_value($object);
                     $collapsed{$name} = ref($value) ? $self->visit($value) : $value;
                 }
             }
@@ -145,29 +150,16 @@ sub compile_collapse_body {
 }
 
 sub compile_expand {
-    my ( $self, $class, $resolver ) = @_;
+    my ( $self, $class, $resolver, @args ) = @_;
 
     my $meta = Class::MOP::get_metaclass_by_name($class);
 
-    my ( %attrs, %lazy );
-
-    my @attrs = grep {
-        !does_role($_->meta, 'KiokuDB::Meta::Attribute::DoNotSerialize')
-            and
-        !does_role($_->meta, 'MooseX::Storage::Meta::Attribute::Trait::DoNotSerialize')
-    } $meta->get_all_attributes;
-
-    foreach my $attr ( @attrs ) {
-        $attrs{$attr->name} = $attr;
-        $lazy{$attr->name}  = does_role($attr->meta, "KiokuDB::Meta::Attribute::Lazy");
-    }
-
-    my $meta_instance = $meta->get_meta_instance;
-
     my $typemap_entry = $self;
 
     my $anon = $meta->is_anon_class;
 
+    my $inner = $self->compile_expand_body($class, $resolver, @args);
+
     return sub {
         my ( $linker, $entry, @args ) = @_;
 
@@ -187,11 +179,51 @@ sub compile_expand {
             return $linker->$method($entry, @args);
         }
 
+        $linker->$inner($entry, @args);
+    }
+}
 
-        my $instance = $meta_instance->create_instance();
+sub compile_create {
+    my ( $self, $class ) = @_;
 
-        # note, this is registered *before* any other value expansion, to allow circular refs
-        $linker->register_object( $entry => $instance );
+    my $meta = Class::MOP::get_metaclass_by_name($class);
+
+    my $meta_instance = $meta->get_meta_instance;
+
+    return sub { $meta_instance->create_instance() };
+}
+
+sub compile_clear {
+    my ( $self, $class ) = @_;
+
+    return sub {
+        my ( $linker, $obj ) = @_;
+        %$obj = (); # FIXME
+    }
+}
+
+sub compile_expand_data {
+    my ( $self, $class, @args ) = @_;
+
+    my $meta = Class::MOP::get_metaclass_by_name($class);
+
+    my $meta_instance = $meta->get_meta_instance;
+
+    my ( %attrs, %lazy );
+
+    my @attrs = grep {
+        !does_role($_->meta, 'KiokuDB::Meta::Attribute::DoNotSerialize')
+            and
+        !does_role($_->meta, 'MooseX::Storage::Meta::Attribute::Trait::DoNotSerialize')
+    } $meta->get_all_attributes;
+
+    foreach my $attr ( @attrs ) {
+        $attrs{$attr->name} = $attr;
+        $lazy{$attr->name}  = does_role($attr->meta, "KiokuDB::Meta::Attribute::Lazy");
+    }
+
+    return sub {
+        my ( $linker, $instance, $entry, @args ) = @_;
 
         my $data = $entry->data;
 
@@ -202,9 +234,9 @@ sub compile_expand {
             my $value = $data->{$name};
 
             if ( ref $value ) {
-                if ( $lazy{$name} and ref($value) ) {
+                if ( $lazy{$name} ) {
                     my $thunk = KiokuDB::Thunk->new( collapsed => $value, linker => $linker, attr => $attr );
-                    $meta_instance->set_slot_value($instance, $attr->name, $thunk); # FIXME low level variant of $attr->set_value
+                    $attr->set_raw_value($instance, $thunk);
                 } else {
                     my @pair = ( $attr, undef );
 
@@ -212,14 +244,15 @@ sub compile_expand {
                     push @values, \@pair;
                 }
             } else {
-                $attr->set_value($instance, $value);
+                $attr->set_raw_value($instance, $value);
             }
         }
 
         $linker->queue_finalizer(sub {
             foreach my $pair ( @values ) {
                 my ( $attr, $value ) = @$pair;
-                $attr->set_value($instance, $value);
+                $attr->set_raw_value($instance, $value);
+                $attr->_weaken_value($instance) if $attr->is_weak_ref;
             }
         });
 
